*Centralized optimization problem for minimizing the surplus energy in a residential area
*3 Types of buildings are considered from the paper "Decentralized optimization approaches for using the load flexibility of electric heating devices":
*buildingy type 1 (HP): non-modulating ground-source heat pump with two storages and constant efficiency (COP)
*buildingy type 2 (mHP): modulating air source heat pump with two storages and not constant efficiency (COP)
*building type 3 (mGas): A gas heating device with two storages and additionally two modulating electic heating elements


*@author: Thomas Dengiz
*Chair of Energy Economics, Karlsruhe Institute of Technology (KIT) in Germany
*Webpage: https://www.iip.kit.edu/english/86_3459.php
*E-Mail: thomas.dengiz@kit.edu



* Define the sets: t_a is the index for the timeslots and household_mHPs, household_HPs and household_mGas are the index of the buildings
Set
    t_a timeslots
    household_mHP household_mHPs
    household_mGas household_mGas
    household_HP household_HPs;


*Read input Data from gdx-file
$GDXIN 'CheckDataSet.gdx';
$LOAD t_a
$LOAD household_mHP
$LOAD household_HP
$LOAD household_mGas

*Set up time markers for measuring the execution time
Scalar time_comp, time_exec, time_elapsed;


* Introduce auxillary time slices in case of an iterative rolling-horizon-optimizaton with more than one optimization horizon
$eval tSlices ceil(card(t_a)/288)
set tSlice          / ts1*ts%tSlices% /
    t(t_a)           'dynamic subset of tt'
    map(tSlice,t_a)  'mapping of time slice to time steps'
;
map(tSlice,t_a) = (ord(tSlice)-1)*288 < ord(t_a) and (ord(tSlice))*288 >= ord(t_a);
option map:0:0:1; display map;
$eval tSlicesHelp ceil(card(t_a)/288)
set tSliceHelp          / ts1*ts%tSlicesHelp% /


*********************************************************************************
* Building type 2: modulating heat pump (mHP)


*Definition of the parameters and reading from gdx-file

Parameter


demand_heating_mHP(t_a, household_mHP)
$LOAD  demand_heating_mHP

demand_electrical_mHP(t_a, household_mHP)
$LOAD  demand_electrical_mHP

volumeOfBufferTank_mHP (household_mHP)
$LOAD volumeOfBufferTank_mHP

volumeOfDHWTank_mHP (household_mHP)
$LOAD volumeOfDHWTank_mHP

electricalPower_mHP (household_mHP)
$LOAD electricalPower_mHP

standingLoss_BufferTank_mHP (household_mHP)
$LOAD standingLoss_BufferTank_mHP

standingLoss_DHWTank_mHP (household_mHP)
$LOAD standingLoss_DHWTank_mHP

demand_DHW_mHP(t_a, household_mHP)
$LOAD demand_DHW_mHP

cop_BufferTank_mHP(t_a, household_mHP)
$LOAD cop_BufferTank_mHP

cop_DHWTank_mHP(t_a, household_mHP)
$LOAD cop_DHWTank_mHP

pv_Nominal_mHP(t_a, household_mHP)
$LOAD pv_Nominal_mHP

pv_Peak_mHP (household_mHP)
$LOAD pv_Peak_mHP

option_relativeGap
$LOAD option_relativeGap

option_absoluteGap
$LOAD option_absoluteGap

option_timeLimit
$LOAD option_timeLimit

 result_in_kWh (tSlice)



*Definition of the variables
positive variable  x_mHP(t_a, household_mHP) heatGenerationForBufferTank;
positive variable y_mHP(t_a, household_mHP) heatGenerationForDHWTank;
binary variable xAct_mHP(t_a, household_mHP);



Variables
         z surplusPowerTotal
         z_correct surplusPowerCorrected;

variable
temperature_level_bufferTank_mHP (t_a, household_mHP)
temperature_level_DHWTank_mHP (t_a, household_mHP);


variable surplusPower(t_a);
positive variable positiveSurplusPower(t_a);
positive variable negativeSurplusPower(t_a);
binary variable isPositive(t_a);
positive variable pvGenerationTotal(t_a);
positive variable electricalPower_Total(t_a);

*Definition of the scalar values
scalar
       DensityOfWater / 988.02/
       SpecificHeatCapacityOfWater /4181/
       DensityOfCement /2400/
       SpecificHeatCapacityOfCement /1000/
       minimalBufferTankTemperature_mHP /20/
       maximalBufferTankTemperature_mHP /22/
       initalBufferTankTemperature_mHP /21/
       minimalDHWTankTemperature_mHP /10/
       maximalDHWTankTemperature_mHP /45/
       initalDHWTankTemperature_mHP /27.5/
       timeResolution in seconds /300/;
       ;
       

*Setting upper and lower limits for the temperature levels of the storages  
temperature_level_DHWTank_mHP.lo(t_a, household_mHP) = minimalDHWTankTemperature_mHP;
temperature_level_DHWTank_mHP.up(t_a, household_mHP) = maximalDHWTankTemperature_mHP;
temperature_level_bufferTank_mHP.lo(t_a, household_mHP) = minimalBufferTankTemperature_mHP;
temperature_level_bufferTank_mHP.up(t_a, household_mHP) = maximalBufferTankTemperature_mHP;

*Fixing the inital temperature levels of the storages
temperature_level_bufferTank_mHP.fx("1", household_mHP) = initalBufferTankTemperature_mHP;
temperature_level_DHWTank_mHP.fx("1", household_mHP) = initalDHWTankTemperature_mHP;


* Maximal values for the decision variables x and y is 1.0
x_mHP.up(t_a, household_mHP) = 1;
y_mHP.up(t_a, household_mHP) = 1;



*Definition of the equations
Equations
          eq_mHP_temperature_level_bufferTank_mHP(t_a, household_mHP)     hot water tank temperature
          eq_mHP_temperature_level_DHWTank_mHP(t_a, household_mHP)     hot water tank temperature
          eq_mHP_onlyOneTankAtOneTime (t_a, household_mHP)         
          equ_surplusPower(t_a)
          equ_pvGenerationTotal(t_a)
          equ_electricalPower_Total(t_a)
          e1(t_a),e2(t_a),e3(t_a)
          eq_mHP_minimalCombinedModulation (t_a, household_mHP)
          eq_mHP_onlyXorY_1 (t_a, household_mHP)
          eq_mHP_onlyXorY_2 (t_a, household_mHP)
          equ_fobj
          eq_z_correct
          eq_mHP_minimalEndTemperature_bufferTank(t_a, household_mHP, tSliceHelp)
          eq_mHP_maximalEndTemperature_bufferTank (t_a, household_mHP, tSliceHelp)
          eq_mHP_minimalEndTemperature_dhwTank(t_a, household_mHP, tSliceHelp)
          eq_mHP_maximalEndTemperature_dhwTank (t_a, household_mHP, tSliceHelp)
          ;

*Constraints for the minimal and maximal temperature at the end of the optimization horizon
eq_mHP_minimalEndTemperature_bufferTank(t(t_a), household_mHP, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_bufferTank_mHP (t_a,household_mHP)
=g=
minimalBufferTankTemperature_mHP + 0.9;


eq_mHP_maximalEndTemperature_bufferTank(t(t_a), household_mHP, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_bufferTank_mHP (t_a,household_mHP)
 =l=
 maximalBufferTankTemperature_mHP - 0.9;


eq_mHP_minimalEndTemperature_dhwTank(t(t_a), household_mHP, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_DHWTank_mHP (t_a,household_mHP)
=g=
minimalDHWTankTemperature_mHP + 15;


eq_mHP_maximalEndTemperature_dhwTank(t(t_a), household_mHP, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_DHWTank_mHP (t_a,household_mHP)
 =l=
 maximalDHWTankTemperature_mHP - 15;
 


*Definition of the energetic difference equation for the temperature of the buffer storage 
eq_mHP_temperature_level_bufferTank_mHP(t(t_a), household_mHP) $(ord(t_a)>1)..
temperature_level_bufferTank_mHP(t_a, household_mHP)
=e=
temperature_level_bufferTank_mHP(t_a-1, household_mHP) + ((x_mHP(t_a, household_mHP) * electricalPower_mHP(household_mHP) * timeResolution *  cop_BufferTank_mHP(t_a, household_mHP) - demand_heating_mHP(t_a, household_mHP)* timeResolution  - standingLoss_BufferTank_mHP(household_mHP) * timeResolution) /((volumeOfBufferTank_mHP(household_mHP)/1000) * DensityOfCement * SpecificHeatCapacityOfCement));


*Definition of the energetic difference equation for the temperature of the hot water tank (from an energy point of view equivalent to having the volume as the variable instead of the temperature)
eq_mHP_temperature_level_DHWTank_mHP(t(t_a), household_mHP) $(ord(t_a)>1)..
temperature_level_DHWTank_mHP(t_a, household_mHP)
=e=
temperature_level_DHWTank_mHP (t_a-1, household_mHP) + ((y_mHP(t_a, household_mHP) * electricalPower_mHP(household_mHP) * timeResolution * cop_DHWTank_mHP(t_a, household_mHP)     - demand_DHW_mHP(t_a, household_mHP) * timeResolution - standingLoss_DHWTank_mHP(household_mHP) * timeResolution) /((volumeOfDHWTank_mHP(household_mHP)/1000) * DensityOfWater * SpecificHeatCapacityOfWater));





*Constraints ensure that only one storage is heated up in each time slot
eq_mHP_onlyOneTankAtOneTime(t, household_mHP)..
y_mHP(t, household_mHP) + x_mHP(t, household_mHP) =l= 1;
eq_mHP_onlyXorY_1(t,household_mHP).. x_mHP(t,household_mHP)    =l= xAct_mHP(t,household_mHP);    
eq_mHP_onlyXorY_2(t,household_mHP).. y_mHP(t,household_mHP)    =l= (1-xAct_mHP(t,household_mHP));

*Constraints for minimal modulation degree of the heat pump prevents the heat pump to switch off and on frequently
eq_mHP_minimalCombinedModulation (t, household_mHP)..
y_mHP(t, household_mHP) + x_mHP(t, household_mHP) =g= 0.1;





*********************************************************************************
* Building type 1: non-modulating heat pump (HP)


*Definition of the parameters and reading from gdx-file
Parameter
demand_heating_HP(t_a, household_HP)
$LOAD  demand_heating_HP

demand_electrical_HP(t_a, household_HP)
$LOAD  demand_electrical_HP

volumeOfBufferTank_HP (household_HP)
$LOAD volumeOfBufferTank_HP

volumeOfDHWTank_HP (household_HP)
$LOAD volumeOfDHWTank_HP

electricalPower_HP (household_HP)
$LOAD electricalPower_HP

standingLoss_BufferTank_HP (household_HP)
$LOAD standingLoss_BufferTank_HP

standingLoss_DHWTank_HP (household_HP)
$LOAD standingLoss_DHWTank_HP

demand_DHW_HP(t_a, household_HP)
$LOAD demand_DHW_HP

cop_BufferTank_HP(t_a, household_HP)
$LOAD cop_BufferTank_HP

cop_DHWTank_HP(t_a, household_HP)
$LOAD cop_DHWTank_HP

pv_Nominal_HP(t_a, household_HP)
$LOAD pv_Nominal_HP

pv_Peak_HP (household_HP)
$LOAD pv_Peak_HP


*Definition of the variables
binary variable  x_HP(t_a, household_HP) heatGenerationForBufferTank;
binary variable y_HP(t_a, household_HP) heatGenerationForDHWTank;
binary variable xAct_HP(t_a, household_HP);

variable
temperature_level_bufferTank_HP (t_a, household_HP)
temperature_level_DHWTank_HP (t_a, household_HP);



*Definition of the scalar values

scalar
       minimalBufferTankTemperature_HP /20/
       maximalBufferTankTemperature_HP /22/
       initalBufferTankTemperature_HP /21/
        minimalDHWTankTemperature_HP /10/
        maximalDHWTankTemperature_HP /45/
       initalDHWTankTemperature_HP /27.5/;
       


*Setting upper and lower limits for the temperature levels of the storages
temperature_level_DHWTank_HP.lo(t_a, household_HP) = minimalDHWTankTemperature_HP;
temperature_level_DHWTank_HP.up(t_a, household_HP) = maximalDHWTankTemperature_HP;
temperature_level_bufferTank_HP.lo(t_a, household_HP) = minimalBufferTankTemperature_HP;
temperature_level_bufferTank_HP.up(t_a, household_HP) = maximalBufferTankTemperature_HP;


*Fixing the inital temperature levels of the storages
temperature_level_bufferTank_HP.fx("1", household_HP) = initalBufferTankTemperature_HP;
temperature_level_DHWTank_HP.fx("1", household_HP) = initalDHWTankTemperature_HP;



*Definition of the equations
Equations
eq_HP_temperature_level_bufferTank_HP(t_a, household_HP)     hot water tank temperature
eq_HP_temperature_level_DHWTank_HP(t_a, household_HP)     hot water tank temperature
eq_HP_onlyOneTankAtOneTime (t_a, household_HP)         
eq_HP_onlyXorY_1 (t_a, household_HP)
eq_HP_onlyXorY_2 (t_a, household_HP)
eq_HP_minimalEndTemperature_bufferTank(t_a, household_HP, tSliceHelp)
eq_HP_maximalEndTemperature_bufferTank (t_a, household_HP, tSliceHelp)
eq_HP_minimalEndTemperature_dhwTank(t_a, household_HP, tSliceHelp)
eq_HP_maximalEndTemperature_dhwTank (t_a, household_HP, tSliceHelp)
;


*Constraints for the minimal and maximal temperature at the end of the optimization horizon

eq_HP_minimalEndTemperature_bufferTank(t(t_a), household_HP, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_bufferTank_HP (t_a,household_HP)
=g=
minimalBufferTankTemperature_HP + 0.9;


eq_HP_maximalEndTemperature_bufferTank(t(t_a), household_HP, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_bufferTank_HP (t_a,household_HP)
 =l=
 maximalBufferTankTemperature_HP - 0.9;


eq_HP_minimalEndTemperature_dhwTank(t(t_a), household_HP, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_DHWTank_HP (t_a,household_HP)
=g=
minimalDHWTankTemperature_HP + 15;


eq_HP_maximalEndTemperature_dhwTank(t(t_a), household_HP, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_DHWTank_HP (t_a,household_HP)
 =l=
 maximalDHWTankTemperature_HP - 15;
 


*Definition of the energetic difference equation for the temperature of the buffer storage
eq_HP_temperature_level_bufferTank_HP(t(t_a), household_HP) $(ord(t_a)>1)..
temperature_level_bufferTank_HP(t_a, household_HP)
=e=
temperature_level_bufferTank_HP (t_a-1, household_HP) + ((x_HP(t_a, household_HP) * electricalPower_HP(household_HP) * timeResolution *  cop_BufferTank_HP(t_a, household_HP) - demand_heating_HP(t_a, household_HP)* timeResolution  - standingLoss_BufferTank_HP(household_HP) * timeResolution) /((volumeOfBufferTank_HP(household_HP)/1000) * DensityOfCement * SpecificHeatCapacityOfCement));


*Definition of the energetic difference equation for the temperature of the hot water tank (from an energy point of view equivalent to having the volume as the variable instead of the temperature)
eq_HP_temperature_level_DHWTank_HP(t(t_a), household_HP) $(ord(t_a)>1)..
temperature_level_DHWTank_HP(t_a, household_HP)
=e=
temperature_level_DHWTank_HP (t_a-1, household_HP) + ((y_HP(t_a, household_HP) * electricalPower_HP(household_HP) * timeResolution * cop_DHWTank_HP(t_a, household_HP)     - demand_DHW_HP(t_a, household_HP) * timeResolution - standingLoss_DHWTank_HP(household_HP) * timeResolution) /((volumeOfDHWTank_HP(household_HP)/1000) * DensityOfWater * SpecificHeatCapacityOfWater));



*Constraints ensure that only one storage is heated up in each time slot
eq_HP_onlyOneTankAtOneTime(t, household_HP)..
y_HP(t, household_HP) + x_HP(t, household_HP) =l= 1;
eq_HP_onlyXorY_1(t,household_HP).. x_HP(t,household_HP)    =l= xAct_HP(t,household_HP);    
eq_HP_onlyXorY_2(t,household_HP).. y_HP(t,household_HP)    =l= (1-xAct_HP(t,household_HP));




*********************************************************************************
* Building type 3: gas heating device with additonally two modulating electric heating elements (mGas)


*Definition of the parameters and reading from gdx-file
Parameter
demand_heating_mGas(t_a, household_mGas)
$LOAD  demand_heating_mGas

demand_electrical_mGas(t_a, household_mGas)
$LOAD  demand_electrical_mGas

volumeOfBufferTank_mGas (household_mGas)
$LOAD volumeOfBufferTank_mGas

volumeOfDHWTank_mGas (household_mGas)
$LOAD volumeOfDHWTank_mGas

electricalPower_SH_mGas (household_mGas)
$LOAD electricalPower_SH_mGas

electricalPower_DHW_mGas (household_mGas)
$LOAD electricalPower_DHW_mGas

gasPower (household_mGas)
$LOAD gasPower


standingLoss_BufferTank_mGas (household_mGas)
$LOAD standingLoss_BufferTank_mGas

standingLoss_DHWTank_mGas (household_mGas)
$LOAD standingLoss_DHWTank_mGas

demand_DHW_mGas(t_a, household_mGas)
$LOAD demand_DHW_mGas

pv_Nominal_mGas(t_a, household_mGas)
$LOAD pv_Nominal_mGas

pv_Peak_mGas (household_mGas)
$LOAD pv_Peak_mGas



*Definition of the variables

positive variable  xElectric_mGas(t_a, household_mGas) heatGenerationForBufferTank;
positive variable yElectric_mGas(t_a, household_mGas) heatGenerationForDHWTank;
binary variable  x_mGas(t_a, household_mGas) heatGenerationForBufferTank;
binary variable y_mGas(t_a, household_mGas) heatGenerationForDHWTank;
binary variable xAct_mGas(t_a, household_mGas);

variable
temperature_level_bufferTank_mGas (t_a, household_mGas)
temperature_level_DHWTank_mGas (t_a, household_mGas);



*Definition of the scalar values
scalar
       minimalBufferTankTemperature_mGas /30/
       maximalBufferTankTemperature_mGas /45/
       initalBufferTankTemperature_mGas /37.5/
        minimalDHWTankTemperature_mGas /10/
        maximalDHWTankTemperature_mGas /45/
       initalDHWTankTemperature_mGas /27.5/
       efficiencyElectricHeating /0.98/
       efficiencyGasHeating /0.98/;
       


*Setting upper and lower limits for the temperature levels of the storages
temperature_level_DHWTank_mGas.lo(t_a, household_mGas) = minimalDHWTankTemperature_mGas;
temperature_level_DHWTank_mGas.up(t_a, household_mGas) = maximalDHWTankTemperature_mGas;
temperature_level_bufferTank_mGas.lo(t_a, household_mGas) = minimalBufferTankTemperature_mGas;
temperature_level_bufferTank_mGas.up(t_a, household_mGas) = maximalBufferTankTemperature_mGas;

*Fixing the inital temperature levels of the storages
temperature_level_bufferTank_mGas.fx("1", household_mGas) = initalBufferTankTemperature_mGas;
temperature_level_DHWTank_mGas.fx("1", household_mGas) = initalDHWTankTemperature_mGas;



*Definition of the equations
Equations
eq_mGas_temperature_level_bufferTank_mGas(t_a, household_mGas)     hot water tank temperature
eq_mGas_temperature_level_DHWTank_mGas(t_a, household_mGas)     hot water tank temperature
eq_mGas_onlyOneTankAtOneTime (t_a, household_mGas)         
eq_mGas_onlyXorY_1 (t_a, household_mGas)
eq_mGas_onlyXorY_2 (t_a, household_mGas)
eq_mGas_minimalEndTemperature_bufferTank(t_a, household_mGas, tSliceHelp)
eq_mGas_maximalEndTemperature_bufferTank (t_a, household_mGas, tSliceHelp)
eq_mGas_minimalEndTemperature_dhwTank(t_a, household_mGas, tSliceHelp)
eq_mGas_maximalEndTemperature_dhwTank (t_a, household_mGas, tSliceHelp)
;

*Constraints for the minimal and maximal temperature at the end of the optimization horizon
eq_mGas_minimalEndTemperature_bufferTank(t(t_a), household_mGas, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_bufferTank_mGas (t_a,household_mGas)
=g=
minimalBufferTankTemperature_mGas + 6;


eq_mGas_maximalEndTemperature_bufferTank(t(t_a), household_mGas, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_bufferTank_mGas (t_a,household_mGas)
 =l=
 maximalBufferTankTemperature_mGas - 6;


eq_mGas_minimalEndTemperature_dhwTank(t(t_a), household_mGas, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_DHWTank_mGas (t_a,household_mGas)
=g=
minimalDHWTankTemperature_mGas + 15;


eq_mGas_maximalEndTemperature_dhwTank(t(t_a), household_mGas, tSliceHelp) $(ord(t_a)=ord(tSliceHelp)*288)..
temperature_level_DHWTank_mGas (t_a,household_mGas)
 =l=
 maximalDHWTankTemperature_mGas - 15;
 


*Definition of the energetic difference equation for the temperature of the buffer storage
eq_mGas_temperature_level_bufferTank_mGas(t(t_a), household_mGas) $(ord(t_a)>1)..
temperature_level_bufferTank_mGas(t_a, household_mGas)
=e=
temperature_level_bufferTank_mGas (t_a-1, household_mGas) + ((x_mGas(t_a, household_mGas) *  gasPower (household_mGas) * efficiencyGasHeating * timeResolution  + xElectric_mGas(t_a, household_mGas) * electricalPower_SH_mGas(household_mGas) * timeResolution * efficiencyElectricHeating - demand_heating_mGas(t_a, household_mGas)* timeResolution  - standingLoss_BufferTank_mGas(household_mGas) * timeResolution) /((volumeOfBufferTank_mGas(household_mGas)/1000) * DensityOfWater * SpecificHeatCapacityOfWater));


*Definition of the energetic difference equation for the temperature of the hot water tank (from an energy point of view equivalent to having the volume as the variable instead of the temperature)
eq_mGas_temperature_level_DHWTank_mGas(t(t_a), household_mGas) $(ord(t_a)>1)..
temperature_level_DHWTank_mGas(t_a, household_mGas)
=e=
temperature_level_DHWTank_mGas (t_a-1, household_mGas) + ((y_mGas(t_a, household_mGas) *  gasPower (household_mGas) * efficiencyGasHeating * timeResolution  + yElectric_mGas(t_a, household_mGas) * electricalPower_DHW_mGas(household_mGas) * timeResolution * efficiencyElectricHeating     - demand_DHW_mGas(t_a, household_mGas) * timeResolution - standingLoss_DHWTank_mGas(household_mGas) * timeResolution) /((volumeOfDHWTank_mGas(household_mGas)/1000) * DensityOfWater * SpecificHeatCapacityOfWater));



*Constraints ensure that only one storage is heated up in each time slot from the gas heating device
eq_mGas_onlyOneTankAtOneTime(t, household_mGas)..
y_mGas(t, household_mGas) + x_mGas(t, household_mGas) =l= 1;
eq_mGas_onlyXorY_1(t,household_mGas).. x_mGas(t,household_mGas)    =l= xAct_mGas(t,household_mGas);    
eq_mGas_onlyXorY_2(t,household_mGas).. y_mGas(t,household_mGas)    =l= (1-xAct_mGas(t,household_mGas));



* Maximal values for the decision variables xElectric and yElectric is 1.0
xElectric_mGas.up(t_a, household_mGas) = 1;
yElectric_mGas.up(t_a, household_mGas) = 1;




****************************************************************************************************
*Combined optimization


*Equations for calculating the total PV generation, the surplus power and the total electrical power
equ_pvGenerationTotal(t).. pvGenerationTotal(t) =e= sum(household_mHP, (pv_Nominal_mHP(t, household_mHP) * pv_Peak_mHP(household_mHP))) + sum(household_HP, (pv_Nominal_HP(t, household_HP) * pv_Peak_HP(household_HP))) + sum(household_mGas, (pv_Nominal_mGas(t, household_mGas) * pv_Peak_mGas(household_mGas)));
equ_surplusPower(t).. surplusPower(t) =e= pvGenerationTotal(t) - electricalPower_Total(t);
equ_electricalPower_Total(t)..electricalPower_Total(t) =e= sum(household_mHP, (x_mHP(t, household_mHP)* electricalPower_mHP(household_mHP) + y_mHP(t,household_mHP) * electricalPower_mHP(household_mHP) + demand_electrical_mHP(t, household_mHP)))
+ sum(household_HP, (x_HP(t, household_HP)* electricalPower_HP(household_HP) + y_HP(t,household_HP) * electricalPower_HP(household_HP) + demand_electrical_HP(t, household_HP)))
+ sum(household_mGas, (xElectric_mGas(t, household_mGas)* electricalPower_SH_mGas(household_mGas) + yElectric_mGas(t,household_mGas) * electricalPower_DHW_mGas(household_mGas) + demand_electrical_mGas(t, household_mGas)));


*Big-M approach for ensuring that only the positive surplus power is minimized 
parameter       
       bigMpos(t_a)
       bigMneg(t_a)  ;
e1(t).. surplusPower(t) =e= positiveSurplusPower(t) - negativeSurplusPower(t);
e2(t).. positiveSurplusPower(t) =l= bigMpos(t) * isPositive(t);
e3(t).. negativeSurplusPower(t) =l= bigMneg(t) * (1-isPositive(t));

bigMpos(t_a) = sum(household_mHP, (pv_Nominal_mHP(t_a, household_mHP) * pv_Peak_mHP(household_mHP))) +  sum(household_HP, (pv_Nominal_HP(t_a, household_HP) * pv_Peak_HP(household_HP))) +  sum(household_mGas, (pv_Nominal_mGas(t_a, household_mGas) * pv_Peak_mGas(household_mGas)));
bigMneg(t_a) = sum(household_mHP, (electricalPower_mHP(household_mHP) + electricalPower_mHP(household_mHP) + demand_electrical_mHP(t_a, household_mHP)))
+ sum(household_HP, (electricalPower_HP(household_HP) + electricalPower_HP(household_HP) + demand_electrical_HP(t_a, household_HP)))
+ sum(household_mGas, (electricalPower_SH_mGas(household_mGas) + electricalPower_DHW_mGas(household_mGas) + demand_electrical_mGas(t_a, household_mGas)));

*Equation for calculating the surplus power without the penalty term
eq_z_correct..
z_correct 
=e=
sum(t,  positiveSurplusPower(t));


*Objective function: minimize sum of surplus power over every time slot --> minimize surplus energy AND slightly punish the use of electricity for heating with a penalty term. This prevents to use electricity instead of gas for heating in time of no generation from renewables 
equ_fobj ..
z
=e=
sum((t, household_mGas),positiveSurplusPower(t) + 0.001* (xElectric_mGas(t, household_mGas)* electricalPower_SH_mGas(household_mGas) + yElectric_mGas(t, household_mGas) * electricalPower_DHW_mGas(household_mGas)));


*Specifying the MIP gaps for the optimizations
Option optcr=0.001;
Option optca=120;
Option Reslim=7200;



Model
     heating_scheduling /all/ ;


*Loop for rolling-horizon-optimization minimzing the objective function and using the last temperature values from the previous iteration as the inital values for the new iteration
loop(tSlice,
  t(t_a) = map(tSlice,t_a); display t;
  Solve heating_scheduling using mip minimizing z;  
  temperature_level_bufferTank_mHP.fx(t_a, household_mHP)$(ord(t_a)=ord(tSlice)*288) =  temperature_level_bufferTank_mHP.l(t_a, household_mHP);
  temperature_level_DHWTank_mHP.fx(t_a, household_mHP)$(ord(t_a)=ord(tSlice)*288) =  temperature_level_DHWTank_mHP.l(t_a, household_mHP);
  temperature_level_bufferTank_HP.fx(t_a, household_HP)$(ord(t_a)=ord(tSlice)*288) =  temperature_level_bufferTank_HP.l(t_a, household_HP);
  temperature_level_DHWTank_HP.fx(t_a, household_HP)$(ord(t_a)=ord(tSlice)*288) =  temperature_level_DHWTank_HP.l(t_a, household_HP);
  temperature_level_bufferTank_mGas.fx(t_a, household_mGas)$(ord(t_a)=ord(tSlice)*288) =  temperature_level_bufferTank_mGas.l(t_a, household_mGas);
  temperature_level_DHWTank_mGas.fx(t_a, household_mGas)$(ord(t_a)=ord(tSlice)*288) =  temperature_level_DHWTank_mGas.l(t_a, household_mGas);
   result_in_kWh (tSlice) = z_correct.L * timeResolution / 3600000;
);

*Calculating the time for the optimization
time_comp = TimeComp;
time_exec = TimeExec;
time_elapsed = TimeElapsed;




